package application

import (
	"erpims/application/service"
	"erpims/config"
	"erpims/oss"
	"git.sxidc.com/go-framework/baize"
	"git.sxidc.com/go-framework/baize/framework/core/api"
	"git.sxidc.com/go-framework/baize/framework/core/application"
	"git.sxidc.com/service-supports/fslog"
	"net/http"
)

var appInstance *application.App

func NewApp() {
	if appInstance != nil {
		return
	}

	appInstance = baize.NewApplication(config.GetConfig().ApplicationConfig)
	//注册OSS
	ossCfg := config.GetConfig().OssConfig
	err := oss.Init(ossCfg.Endpoint, ossCfg.KeyID, ossCfg.KeySecret, ossCfg.Prefix)
	if err != nil {
		fslog.Error(err)
		panic(err)
	}

	// 注册Router
	appInstance.Api().PrefixRouter().RegisterVersionedRouter("v1")

}

func DestroyApp() {
	if appInstance == nil {
		return
	}

	baize.DestroyApplication(appInstance)
	appInstance = nil
}

func Start() error {
	// 初始化服务
	for _, svc := range service.RegisteredServices {
		err := svc.Init(appInstance)
		if err != nil {
			return err
		}
	}

	err := appInstance.Start()
	if err != nil {
		return err
	}

	return nil
}

func Finish() error {
	err := appInstance.Finish()
	if err != nil {
		return err
	}

	// 销毁服务
	for _, svc := range service.RegisteredServices {
		err := svc.Destroy()
		if err != nil {
			return err
		}
	}

	return nil
}

func ServerHttpForTest(w http.ResponseWriter, req *http.Request) {
	appInstance.Api().RootRouter().(*api.RootRouter).ServerHttp(w, req)
}
