package about_us

import (
	"git.sxidc.com/go-framework/baize/framework/core/domain"
	"git.sxidc.com/go-framework/baize/framework/core/domain/entity"
)

const (
	FieldCompanyTel        = "CompanyTel"
	FieldOnlineShoppingTel = "OnlineShoppingTel"
	FieldAfterSaleTel      = "AfterSaleTel"
	FieldCompanyAddress    = "CompanyAddress"
	FieldPostCode          = "PostCode"
	FieldFax               = "Fax"
	FieldEmail             = "Email"
	FieldWxMpQrCode        = "WxMpQrCode"
	FieldCopyright         = "Copyright"
	FieldRecordNumber      = "RecordNumber"
	FieldContactQrCode     = "ContactQrCode"
	FieldContent           = "Content"
)

var fieldMap = map[string]string{
	FieldCompanyTel:        "公司电话",
	FieldOnlineShoppingTel: "网购热线",
	FieldAfterSaleTel:      "访问路径",
	FieldCompanyAddress:    "公司地址",
	FieldPostCode:          "邮编",
	FieldFax:               "传真",
	FieldEmail:             "邮箱",
	FieldWxMpQrCode:        "微信公众号二维码",
	FieldCopyright:         "版权所有",
	FieldRecordNumber:      "备案号",
	FieldContactQrCode:     "联系方式二维码",
	FieldContent:           "内容",
}

var (
	ColumnCompanyTel        = domain.ColumnName(FieldCompanyTel)
	ColumnOnlineShoppingTel = domain.ColumnName(FieldOnlineShoppingTel)
	ColumnAfterSaleTel      = domain.ColumnName(FieldAfterSaleTel)
	ColumnCompanyAddress    = domain.ColumnName(FieldCompanyAddress)
	ColumnPostCode          = domain.ColumnName(FieldPostCode)
	ColumnFax               = domain.ColumnName(FieldFax)
	ColumnEmail             = domain.ColumnName(FieldEmail)
	ColumnWxMpQrCode        = domain.ColumnName(FieldWxMpQrCode)
	ColumnCopyright         = domain.ColumnName(FieldCopyright)
	ColumnRecordNumber      = domain.ColumnName(FieldRecordNumber)
	ColumnContactQrCode     = domain.ColumnName(FieldContactQrCode)
	ColumnContent           = domain.ColumnName(FieldContent)
)

type Entity struct {
	entity.Base
	CompanyTel        string `sqlmapping:"column:company_tel;" sqlresult:"column:company_tel;" check:"required,lte=32,when=create/update"`
	OnlineShoppingTel string `sqlmapping:"column:online_shopping_tel;" sqlresult:"column:online_shopping_tel;" check:"required,lte=32,when=create/update"`
	AfterSaleTel      string `sqlmapping:"column:after_sale_tel;" sqlresult:"column:after_sale_tel;" check:"required,lte=32,when=create/update"`
	CompanyAddress    string `sqlmapping:"column:company_address;" sqlresult:"column:company_address;" check:"required,lte=256,when=create/update"`
	PostCode          string `sqlmapping:"column:post_code;" sqlresult:"column:post_code;" check:"required,lte=32,when=create/update"`
	Fax               string `sqlmapping:"column:fax;" sqlresult:"column:fax;" check:"required,lte=32,when=create/update"`
	Email             string `sqlmapping:"column:email;" sqlresult:"column:email;" check:"required,lte=32,when=create/update"`
	WxMpQrCode        string `sqlmapping:"column:wx_mp_qr_code;" sqlresult:"column:wx_mp_qr_code;" check:"required,lte=256,when=create/update"`
	Copyright         string `sqlmapping:"column:copyright;" sqlresult:"column:copyright;" check:"required,lte=128,when=create/update"`
	RecordNumber      string `sqlmapping:"column:record_number;" sqlresult:"column:record_number;" check:"required,lte=128,when=create/update"`
	PubRecordNumber   string `sqlmapping:"column:pub_record_number;" sqlresult:"column:pub_record_number;" check:"required,lte=128,when=create/update"`
	ContactQrCode     string `sqlmapping:"column:contact_qr_code;updateClear;" sqlresult:"column:contact_qr_code;"`
	Content           string `sqlmapping:"column:content;" sqlresult:"column:content;" check:"required,when=create/update"`
	entity.UserIDFields
	entity.OperatorUserNameField
	entity.TimeFields

	/* HAC: ENTITY RELATION FIELDS */
	/* HAC: END ENTITY RELATION FIELDS */
}

func (e *Entity) DomainCNName() string {
	return "关于我们"
}

func (e *Entity) DomainCamelName() string {
	return "AboutUs"
}

func (e *Entity) GetFieldMap() map[string]string {
	return fieldMap
}

/* HAC: ENTITY RELATION METHODS */
/* HAC: END ENTITY RELATION METHODS */
