package announcement_policy_info

import (
	"git.sxidc.com/go-framework/baize/framework/core/domain"
	"git.sxidc.com/go-framework/baize/framework/core/domain/entity"
	"time"
)

const (
	FieldTitle           = "Title"
	FieldFirstLevelType  = "FirstLevelType"
	FieldSecondLevelType = "SecondLevelType"
	FieldReleaseDate     = "ReleaseDate"
	FieldTitleImgUrl     = "TitleImgUrl"
	FieldContent         = "Content"
)

var fieldMap = map[string]string{
	FieldTitle:           "标题",
	FieldFirstLevelType:  "类型 1:企业公告  2：采购公告 3：经销政策",
	FieldSecondLevelType: "企业公告类型 1:规章制度 2：企业动态",
	FieldReleaseDate:     "发布日期",
	FieldTitleImgUrl:     "标题图片引用路径",
	FieldContent:         "内容",
}

var (
	ColumnTitle           = domain.ColumnName(FieldTitle)
	ColumnFirstLevelType  = domain.ColumnName(FieldFirstLevelType)
	ColumnSecondLevelType = domain.ColumnName(FieldSecondLevelType)
	ColumnReleaseDate     = domain.ColumnName(FieldReleaseDate)
	ColumnTitleImgUrl     = domain.ColumnName(FieldTitleImgUrl)
	ColumnContent         = domain.ColumnName(FieldContent)
)

type Entity struct {
	entity.Base
	Title           string     `sqlmapping:"column:title;" sqlresult:"column:title;" check:"required,lte=256,when=create/update"`
	FirstLevelType  int        `sqlmapping:"column:first_level_type;" sqlresult:"column:first_level_type;" check:"required,when=create/update"`
	SecondLevelType int        `sqlmapping:"column:second_level_type;updateClear;" sqlresult:"column:second_level_type;"`
	ReleaseDate     *time.Time `sqlmapping:"column:release_date;" sqlresult:"column:release_date;" check:"required,when=create/update"`
	TitleImgUrl     string     `sqlmapping:"column:title_img_url;updateClear;" sqlresult:"column:title_img_url;"`
	Content         string     `sqlmapping:"column:content;updateClear;" sqlresult:"column:content;"`
	entity.UserIDFields
	entity.OperatorUserNameField
	entity.TimeFields

	/* HAC: ENTITY RELATION FIELDS */
	/* HAC: END ENTITY RELATION FIELDS */
}

func (e *Entity) DomainCNName() string {
	return "公告政策信息"
}

func (e *Entity) DomainCamelName() string {
	return "AnnouncementPolicyInfo"
}

func (e *Entity) GetFieldMap() map[string]string {
	return fieldMap
}

/* HAC: ENTITY RELATION METHODS */
/* HAC: END ENTITY RELATION METHODS */
