package common_use_website

import (
	"git.sxidc.com/go-framework/baize/framework/core/domain"
	"git.sxidc.com/go-framework/baize/framework/core/domain/entity"
)

const (
	FieldTitle        = "Title"
	FieldIconUrl      = "IconUrl"
	FieldAccessUrl    = "AccessUrl"
	FieldSort         = "Sort"
	FieldExternalLink = "ExternalLink"
)

var fieldMap = map[string]string{
	FieldTitle:        "标题",
	FieldIconUrl:      "图标路径",
	FieldAccessUrl:    "访问路径",
	FieldSort:         "排序",
	FieldExternalLink: "是否外部链接",
}

var (
	ColumnTitle        = domain.ColumnName(FieldTitle)
	ColumnIconUrl      = domain.ColumnName(FieldIconUrl)
	ColumnAccessUrl    = domain.ColumnName(FieldAccessUrl)
	ColumnSort         = domain.ColumnName(FieldSort)
	ColumnExternalLink = domain.ColumnName(FieldExternalLink)
)

type Entity struct {
	entity.Base
	Title        string `sqlmapping:"column:title;" sqlresult:"column:title;" check:"required,lte=256,when=create/update"`
	IconUrl      string `sqlmapping:"column:icon_url;" sqlresult:"column:icon_url;" check:"required,lte=256,when=create/update"`
	AccessUrl    string `sqlmapping:"column:access_url;" sqlresult:"column:access_url;" check:"required,lte=256,when=create/update"`
	Sort         int    `sqlmapping:"column:sort;" sqlresult:"column:sort;" check:"required,when=create/update"`
	ExternalLink int    `sqlmapping:"column:external_link;" sqlresult:"column:external_link;" check:"required,when=create/update"`
	entity.UserIDFields
	entity.OperatorUserNameField
	entity.TimeFields

	/* HAC: ENTITY RELATION FIELDS */
	/* HAC: END ENTITY RELATION FIELDS */
}

func (e *Entity) DomainCNName() string {
	return "常用网站"
}

func (e *Entity) DomainCamelName() string {
	return "CommonUseWebsite"
}

func (e *Entity) GetFieldMap() map[string]string {
	return fieldMap
}

/* HAC: ENTITY RELATION METHODS */
/* HAC: END ENTITY RELATION METHODS */
