package service

import (
	"erpims/application/domain/common_use_website"
	"erpims/oss"
	"git.sxidc.com/go-framework/baize/convenient/entity_crud"
	"git.sxidc.com/go-framework/baize/framework/binding"
	"git.sxidc.com/go-framework/baize/framework/core/api"
	"git.sxidc.com/go-framework/baize/framework/core/api/request"
	"git.sxidc.com/go-framework/baize/framework/core/api/response"
	"git.sxidc.com/go-framework/baize/framework/core/application"
	"git.sxidc.com/go-framework/baize/framework/core/domain"
	"git.sxidc.com/go-framework/baize/framework/core/domain/entity"
	"git.sxidc.com/go-framework/baize/framework/core/infrastructure"
	"git.sxidc.com/go-framework/baize/framework/core/infrastructure/database"
	"git.sxidc.com/go-framework/baize/framework/core/infrastructure/database/sql"
	"git.sxidc.com/go-tools/utils/strutils"
	"github.com/pkg/errors"
)

var commonUseWebsiteService = &CommonUseWebsiteService{}

type CommonUseWebsiteService struct{}

func (svc *CommonUseWebsiteService) Init(appInstance *application.App) error {
	svc.v1(appInstance)
	return nil
}

func (svc *CommonUseWebsiteService) Destroy() error {
	return nil
}

func (svc *CommonUseWebsiteService) v1(appInstance *application.App) {
	v1Binder := binding.NewBinder(appInstance.ChooseRouter(api.RouterPrefix, "v1"),
		appInstance.Infrastructure())

	entity_crud.BindSimple[common_use_website.Info](v1Binder,
		&entity_crud.Simple[common_use_website.Info]{
			Entity:             &common_use_website.Entity{},
			Schema:             dbSchema,
			CreateJsonBody:     &common_use_website.CreateCommonUseWebsiteJsonBody{},
			DeleteQueryParams:  &common_use_website.DeleteCommonUseWebsiteQueryParams{},
			UpdateJsonBody:     &common_use_website.UpdateCommonUseWebsiteJsonBody{},
			QueryQueryParams:   &common_use_website.GetCommonUseWebsitesQueryParams{},
			GetByIDQueryParams: &common_use_website.GetCommonUseWebsiteQueryParams{},
		},
		entity_crud.WithCreateTx(),
		entity_crud.WithCreateCallbacks(&entity_crud.CreateCallbacks{
			Before: func(c *api.Context, params request.Params, e entity.Entity, prepared map[string]any, i *infrastructure.Infrastructure, tx database.Executor) error {
				requestParams, err := request.ToConcrete[*common_use_website.CreateCommonUseWebsiteJsonBody](params)
				if err != nil {
					return err
				}
				info, err := domain.ToConcrete[*common_use_website.Entity](e)
				if err != nil {
					return err
				}

				//图片处理
				if strutils.IsStringNotEmpty(requestParams.IconUrl) {
					newPicture, err := oss.MultipleFileMv(requestParams.IconUrl)
					if err != nil {
						return err
					}
					info.IconUrl = newPicture
				}

				return nil
			},
		}),
		entity_crud.WithUpdateTx(),
		entity_crud.WithUpdateCallbacks(&entity_crud.UpdateCallbacks{
			Before: func(c *api.Context, params request.Params, e entity.Entity, prepared map[string]any, i *infrastructure.Infrastructure, tx database.Executor) error {
				requestParams, err := request.ToConcrete[*common_use_website.UpdateCommonUseWebsiteJsonBody](params)
				info, err := domain.ToConcrete[*common_use_website.Entity](e)
				if err != nil {
					return err
				}

				if strutils.IsStringNotEmpty(requestParams.IconUrl) {
					newPicture, err := oss.MultipleFileMv(requestParams.IconUrl)
					if err != nil {
						return err
					}
					info.IconUrl = newPicture
				}
				return nil
			},
		}),
		entity_crud.WithQueryCallbacks(&entity_crud.QueryCallbacks[common_use_website.Info]{
			OnSuccessReturn: func(c *api.Context, params request.Params, e entity.Entity, i *infrastructure.Infrastructure, output response.InfosData[common_use_website.Info]) (response.InfosData[common_use_website.Info], error) {
				for i2 := range output.Infos {
					item := &output.Infos[i2]
					if strutils.IsStringNotEmpty(item.IconUrlStr) {
						fileObjects, err := oss.MultipleFileUrl(item.IconUrlStr)
						if err != nil {
							return output, err
						}
						item.IconUrl = fileObjects
					}
				}
				return output, nil
			},
		}),
		entity_crud.WithGetByIDCallbacks(&entity_crud.GetByIDCallbacks[common_use_website.Info]{
			OnSuccessReturn: func(c *api.Context, params request.Params, e entity.Entity, i *infrastructure.Infrastructure, output common_use_website.Info) (common_use_website.Info, error) {
				if strutils.IsStringNotEmpty(output.IconUrlStr) {
					fileObjects, err := oss.MultipleFileUrl(output.IconUrlStr)
					if err != nil {
						return output, err
					}
					output.IconUrl = fileObjects
				}

				return output, nil
			},
		}),
		entity_crud.WithQueryOrderBy[common_use_website.Info](common_use_website.ColumnSort+" asc"),
		entity_crud.WithQueryConditionFieldCallback[common_use_website.Info](func(conditions *sql.Conditions, fieldName string, columnName string, value any) (hasDeal bool, err error) {
			switch fieldName {
			case common_use_website.FieldTitle:
				conditions.Like(columnName, "%"+value.(string)+"%")
				return true, nil
			case common_use_website.FieldAccessUrl:
				conditions.Like(columnName, "%"+value.(string)+"%")
				return true, nil
			default:
				return false, nil
			}
		}))
}

func (svc *CommonUseWebsiteService) queryByKeyFields(e *common_use_website.Entity, dbExecutor database.Executor) (*common_use_website.Entity, error) {
	result, err := database.QueryOne(dbExecutor, &sql.QueryOneExecuteParams{
		TableName:  domain.TableName(dbSchema, e),
		Conditions: sql.NewConditions().Equal(entity.ColumnID, e.ID),
	})
	if err != nil {
		if database.IsErrorDBRecordNotExist(err) {
			return nil, errors.New(e.DomainCNName() + "不存在")
		}

		return nil, err
	}

	existEntity := new(common_use_website.Entity)
	err = sql.ParseSqlResult(result, existEntity)
	if err != nil {
		return nil, err
	}

	return existEntity, nil
}

func (svc *CommonUseWebsiteService) checkExistByKeyFields(e *common_use_website.Entity, dbExecutor database.Executor) error {
	exist, err := database.CheckExist(dbExecutor, &sql.CheckExistExecuteParams{
		TableName:  domain.TableName(dbSchema, e),
		Conditions: sql.NewConditions().Equal(entity.ColumnID, e.ID),
	})
	if err != nil {
		return err
	}

	if !exist {
		return errors.New(e.DomainCNName() + "不存在")
	}

	return nil
}
